#!/usr/bin/ruby -Ke

# ATN Visualizer with Graphviz
# by Keichiro Katayama

# version 1.0alpha

# ChangeLog
#  1.0 2004/06/27 First Release
#  1.1 2004/06/28 Split Multi Postscript

# =====================================
# Start Up
# =====================================
require 'graphviz'


# =====================================
# Functions
# =====================================
def show_usage
  $stderr.puts "usage: #{$0} <automaton file> <dest-dir>"
end

def draw_graph( original_name, output_path, graph_name_raw, arc_list, output_type = 'ps' )
  graph_name = graph_name_raw.gsub( /[-\.]/, '_' )
  graph_name.gsub!( /\//, '' )
  if graph_name.empty? then
    graph_name = 'any'
  end
  output_file = output_path + '/' + File::basename( original_name ) + '-' + graph_name + '.ps'
#p graph_name
#p output_file
#return
  # GraphViz Setup
  gv = GraphViz::new( File::basename(graph_name).downcase,
                      'output' => output_type,
                      'file' => output_file )

  # Node Attributes
  gv.node['shape'] = 'circle'
  gv.node['color'] = 'black'
  gv.node['group'] = graph_name.downcase
  gv.node['fontname'] = 'Times'

  # Edge Attributes
  gv.edge['color'] = 'black'
  gv.edge['fontname'] = 'Helvetica'

  # Graph Attributes
  gv['rankdir'] = 'LR'
  gv['rank'] = 'sink'
  gv['center'] = 'true'

  gv['color'] = 'black'
  #gv['concentrate'] = 'true'
  gv['size'] = '7,10.5'
  gv['ratio'] = 'auto'

  gv['label'] = graph_name_raw
  gv['labeljust'] = 'l' # Left-side
  gv['labelloc'] = 't'  # Top
  gv['fontname'] = 'Helvetica'

  # Check Node List
  nodes = Array::new
  arc_list.each { |arc_data|
    nodes << arc_data[0] # Start Node
    nodes << arc_data[1] # Dest Node
  }
  nodes.sort!
  nodes.uniq!

  # Add Nodes
  node_objs = Array::new
  nodes.each{ |node_no|
    case node_no
    when 0 # End of Automaton
      node_objs[node_no] = gv.add_node( "#{graph_name}__#{node_no}", 'label' => '', 'shape' => 'plaintext' )
    when 1 # Start
      node_objs[node_no] = gv.add_node( "#{graph_name}__#{node_no}", 'label' => node_no.to_s, 'style' => 'filled', 'fillcolor' => 'black', 'fontcolor' => 'white' )
    else
      node_objs[node_no] = gv.add_node( "#{graph_name}__#{node_no}", 'label' => node_no.to_s )
    end
  }

  # Add Edges
  arc_list.sort.each{ |arc_data|
    start = node_objs[ arc_data[0] ]
    dest  = node_objs[ arc_data[1] ]
    if arc_data[2][0] < 128 then
      gv.add_edge( start, dest, 'label' => arc_data[2] ) 
    else
      gv.add_edge( start, dest, 'label' => arc_data[2], 'fontname' => 'GothicBBB-Medium-EUC-H')
    end
  }

  gv.output
end


# =====================================
# Main
# =====================================

# Parameter Check

if ARGV.length < 2 then
  show_usage
  exit
end
target = ARGV.shift
dest_dir = ARGV.shift
if ! File::exists?( target ) || !  File::file?( target ) then
  $stderr.puts "Cannot found #{target}"
  exit
end
if ! File::exists?( dest_dir ) || ! File::directory?( dest_dir ) then
  $stderr.puts "Cannot found #{dest_dir}"
  exit
end

# Parse Automaton
atn = File::open( target, 'r' )

arc_list = nil
symbol_name = ''
atn.each{ |line|
  line.chomp!
  line.strip!

  # Comments, Empty Line
  if line =~ /^;/ || line.empty? then
    next
  end

  # End of define
  if line == 'end' then
    if ! arc_list.empty? then
      draw_graph( target, dest_dir, symbol_name, arc_list )
      arc_list = nil
    end
    next
  end

  # Start of define
  if line =~ /^(\/[ytcfk]*|[A-Z]+)$/ then
    arc_list = Array::new
    symbol_name = line
    next
  end

  # Add arc data
  arc_list += line.scan( /arc\((\d+,\d+,[^\)]+)\)/ ).collect { |arc_define|
    tmp = arc_define.first.split( ',', 3 )
    tmp[0] = tmp[0].to_i
    tmp[1] = tmp[1].to_i
    tmp[2].sub!( /,/, '' )
    tmp
  }
}
atn.close

